C     mzap -- cleans Doppler data
C     
C     Original mex file by Danny Chapman
C
C     Incorporated into the chil code by Robin Hogan from 2 Nov 2001
C
C============== MZAP SUBROUTINE ================================
C vel - output velocity field
C v - original velocity data
C maxvar - maximum variance allowed, calculated from the
C     surrounding points
C lonely1 - if number of NaNs surrounding an unremoved data point
C     is more than lonely1 then that data point is removed (lonely1=8
C     has no effect
C lonely2 - if number of NaNs surrounding point is less than
C     lonely2, then removed point is replaced (to prevent holes in
C     thedata: lonely2=0 has no effect
C [mm,nn] - the size of the arrays vel, region and v_orig
C vf - folding velocity goes from -vf to +vf
C nan - value to indicate no data
C===============================================================
      subroutine mzap(vel, v, maxvar, lonely1, lonely2, mm, nn, vf, nan)
      IMPLICIT NONE
      integer*4 mm, nn, M, N
      integer*4 i, j, k

      PARAMETER (M=400, N=1600)
      real*4 maxvar, lonely1, lonely2
      real*4 vel(mm, nn), v(mm, nn)

      real*4 mv2, var(M,N), nan, vf
      integer mask1(M,N)
      
      mv2=maxvar**2

C work out the variance matrix

      IF ((mm.LE.M).AND.(nn.LE.N)) THEN

         DO i=2,mm-1
            DO j=2,nn-1
               var(i,j)=(((v(i,j-1)-v(i,j))**2)+((v(i,j-1)-v(i,j))**2)
     $              +((v(i+1,j-1)-v(i,j))**2)+((v(i-1,j)-v(i,j))**2)
     $              +((v(i+1,j)-v(i,j))**2)+((v(i-1,j+1)-v(i,j))**2)
     $              +((v(i,j+1)-v(i,j))**2)+((v(i+1,j-1)-v(i,j))**2))
     $              /8.0
            ENDDO
         ENDDO
         
         DO i=2,mm-1 
            var(i,nn)=(((v(i-1,nn)-v(i,nn))**2)+((v(i-1,nn-1)- v(i,nn)
     $           )**2)+((v(i,nn-1)-v(i,nn))**2)+((v(i+1,nn-1)- v(i,nn)
     $           )**2)+((v(i+1,nn)-v(i,nn))**2))/5.0

            var(i,1)=(((v(i-1,1)-v(i,1))**2)+((v(i-1,2)-v(i,1))**2)+
     $           ((v(i,2)-v(i,1))**2)+((v(i+1,2)-v(i,1))**2)+((v(i+1,1
     $           )-v(i,1))**2))/5.0
         ENDDO

         DO j=2,nn-1 
            var(1,j)=(((v(1,j-1)-v(1,j))**2)+((v(2,j-1)-v(1,j))**2)+
     $           ((v(2,j)-v(1,j))**2)+((v(2,j+1)-v(1,j))**2)+((v(1,j+1
     $           )-v(1,j))**2))/5.0

            var(mm,j)=(((v(mm,j-1)-v(mm,j))**2)+((v(mm-1,j-1)- v(mm,j)
     $           )**2)+((v(mm-1,j)-v(mm,j))**2)+((v(mm-1,j+1)- v(mm,j)
     $           )**2)+((v(mm,j+1)-v(mm,j))**2))/5.0
         ENDDO
         
C     now do the corners - what a pain
         var(1,1)=(((v(2,1)-v(1,1))**2)+((v(1,2)-v(1,1))**2)+ ((v(2,2)
     $        -v(1,1))**2))/3.0
         var(mm,1)=(((v(mm-1,1)-v(mm,1))**2)+((v(1,2)-v(mm,1))**2)+
     $        ((v(mm-1,2)-v(mm,1))**2))/3.0
         var(1,nn)=(((v(2,nn)-v(1,nn))**2)+((v(1,nn-1)-v(1,nn))**2)+
     $        ((v(2,nn-1)-v(1,nn))**2))/3.0
         var(mm,nn)=(((v(mm-1,nn)-v(mm,nn))**2)+((v(mm,nn-1)- v(mm,nn)
     $        )**2)+((v(mm-1,nn-1)-v(mm,nn))**2))/3.0

C     create the mask, and prepare v, vel for rotated calc
         
         DO i=1,mm
            DO j=1,nn
               IF (var(i,j).LT.mv2) THEN
                  mask1(i,j)=1
               ELSE
                  mask1(i,j)=0
               ENDIF
               vel(i,j)=v(i,j)+vf
               IF (vel(i,j).GT.vf) THEN
                  vel(i,j)=vel(i,j)-vf*2.0
               ENDIF
            ENDDO
         ENDDO

C     work out the variance matrix for the rotated v

         DO i=2,mm-1
            DO j=2,nn-1
               var(i,j)=(((vel(i,j-1)-vel(i,j))**2)+((vel(i,j-1)-vel(i
     $              ,j))**2)+((vel(i+1,j-1)-vel(i,j))**2)+((vel(i-1,j)
     $              -vel(i,j))**2)+((vel(i+1,j)-vel(i,j))**2)+((vel(i
     $              -1,j+1)-vel(i,j))**2)+((vel(i,j+1)-vel(i,j))**2)
     $              +((vel(i+1,j-1)-vel(i,j))**2))/8.0
            ENDDO
         ENDDO
         
         DO i=2,mm-1 
            var(i,nn)=(((vel(i-1,nn)-vel(i,nn))**2)+((vel(i-1,nn-1)-
     $           vel(i,nn))**2)+((vel(i,nn-1)-vel(i,nn))**2)+((vel(i+1
     $           ,nn-1)-vel(i,nn))**2)+((vel(i+1,nn)-vel(i,nn))**2))/5
     $           .0

            var(i,1)=(((vel(i-1,1)-vel(i,1))**2)+((vel(i-1,2)-vel(i,1)
     $           )**2)+((vel(i,2)-vel(i,1))**2)+ ((vel(i+1,2)-vel(i,1)
     $           )**2)+((vel(i+1,1)-vel(i,1))**2))/5.0
         ENDDO

         DO j=2,nn-1 
            var(1,j)=(((vel(1,j-1)-vel(1,j))**2)+ ((vel(2,j-1)-vel(1,j
     $           ))**2)+ ((vel(2,j)-vel(1,j))**2)+((vel(2,j+1)-vel(1,j
     $           ))**2) +((vel(1,j+1)-vel(1,j))**2))/5.0

            var(mm,j)=(((vel(mm,j-1)-vel(mm,j))**2)+((vel(mm-1,j-1)-
     $           vel(mm,j))**2)+((vel(mm-1,j)-vel(mm,j))**2)+((vel(mm
     $           -1,j+1)-vel(mm,j))**2)+((vel(mm,j+1)-vel(mm,j))**2))
     $           /5.0
         ENDDO
         
C     now do the corners - what a pain
         var(1,1)=(((vel(2,1)-vel(1,1))**2)+((vel(1,2)-vel(1,1))**2)+
     $        ((vel(2,2)-vel(1,1))**2))/3.0
         var(mm,1)=(((vel(mm-1,1)-vel(mm,1))**2) +((vel(1,2)-vel(mm,1)
     $        )**2)+((vel(mm-1,2)-vel(mm,1))**2)) /3.0
         var(1,nn)=(((vel(2,nn)-vel(1,nn))**2)+ ((vel(1,nn-1)-vel(1,nn
     $        ))**2)+((vel(2,nn-1)-vel(1,nn))**2))/3.0
         var(mm,nn)=(((vel(mm-1,nn)-vel(mm,nn))**2)+((vel(mm,nn-1)-
     $        vel(mm,nn))**2)+((vel(mm-1,nn-1)-vel(mm,nn))**2))/3.0

C     create the second mask, and create composite, requiring
C     discontinuity in both velocity fields
         
         DO i=1,mm
            DO j=1,nn
               vel(i,j)=nan
               IF ((var(i,j).LT.mv2).OR.(mask1(i,j).EQ.1)) THEN
                  vel(i,j)=v(i,j)
               ENDIF
            ENDDO
         ENDDO

C     check for lonely points in data sparse regions
         DO i=2,mm-1
            DO j=2,nn-1
               mask1(i,j)=((vel(i,j).NE.nan).AND.(((vel(i-1,j).EQ.nan) 
     $              +(vel(i-1,j+1).EQ.nan)+(vel(i,j+1).EQ.nan)
     $              +(vel(i+1,j+1).EQ.nan)+(vel(i+1,j).EQ.nan)
     $              +(vel(i+1,j-1).EQ.nan)+(vel(i,j-1).EQ.nan)
     $              +(vel(i-1,j-1).EQ.nan)).GT.lonely1))
            ENDDO
         ENDDO

         i=1
         k=mm
         DO j=2,nn-1
            mask1(i,j)=((vel(i,j).NE.nan).AND.(((vel(i,j+1).EQ.nan)
     $           +(vel(i+1,j+1).EQ.nan)+(vel(i+1,j).EQ.nan)
     $           +(vel(i+1,j-1).EQ.nan)+(vel(i,j-1).EQ.nan)).GT
     $           .(lonely1*5/8)))
            mask1(k,j)=((vel(k,j).NE.nan).AND.(((vel(k-1,j).EQ.nan)
     $           +(vel(k-1,j+1).EQ.nan)+(vel(k,j+1).EQ.nan)
     $           +(vel(k,j-1).EQ.nan)+(vel(k-1,j-1).EQ.nan)).GT
     $           .(lonely1*5/8)))
         ENDDO

         j=1
         k=nn
         DO i=2,mm-1
            mask1(i,j)=((vel(i,j).NE.nan).AND.(((vel (i-1,j).EQ.nan)
     $           +(vel(i-1,j+1).EQ.nan)+(vel(i,j+1).EQ.nan)
     $           +(vel(i+1,j+1).EQ.nan)+(vel(i+1,j).EQ.nan)).GT
     $           .(lonely1*5/8)))
            mask1(i,k)=((vel(i,k).NE.nan).AND.(((vel (i-1,k).EQ.nan)
     $           +(vel(i+1,k).EQ.nan)+(vel(i+1,k-1).EQ.nan)
     $           +(vel(i,k-1).EQ.nan)+(vel(i-1,k-1).EQ.nan)).GT
     $           .(lonely1*5/8)))
         ENDDO

         DO i=1,mm
            DO j=1,nn
               IF (mask1(i,j).EQ.1) THEN
                  vel(i,j)=nan
               ENDIF
            ENDDO
         ENDDO

         DO i=2,mm-1
            DO j=2,nn-1
               mask1(i,j)=((vel(i,j).EQ.nan).AND.(((vel(i-1,j).EQ.nan)
     $              +(vel(i-1,j+1).EQ.nan) +(vel(i,j+1).EQ.nan)
     $              +(vel(i+1,j+1).EQ.nan) +(vel(i+1,j).EQ.nan)
     $              +(vel(i+1,j-1).EQ.nan) +(vel(i,j-1).EQ.nan)
     $              +(vel(i-1,j-1))).LT.lonely2))
            ENDDO
         ENDDO

         i=1
         k=mm
         DO j=2,nn-1
            mask1(i,j)=((vel(i,j).EQ.nan).AND.(((vel(i ,j+1).EQ.nan)
     $           +(vel(i+1,j+1).EQ.nan)+(vel(i+1,j).EQ.nan)
     $           +(vel(i+1,j-1).EQ.nan)+(vel(i,j-1).EQ.nan)).LT
     $           .(lonely2*5/8)))
            mask1(k,j)=((vel(k,j).EQ.nan).AND.(((vel(k-1,j).EQ.nan)
     $           +(vel(k-1,j+1).EQ.nan)+(vel(k,j+1).EQ.nan)
     $           +(vel(k,j-1).EQ.nan)+(vel(k-1,j-1)).EQ.nan).LT
     $           .(lonely2*5/8)))
         ENDDO

         k=nn
         j=1
         DO i=2,mm-1
            mask1(i,j)=((vel(i,j).EQ.nan).AND.(((vel(i -1,j).EQ.nan)
     $           +(vel(i-1,j+1).EQ.nan)+(vel(i,j+1).EQ.nan)
     $           +(vel(i+1,j+1).EQ.nan)+(vel(i+1,j).EQ.nan)).LT
     $           .(lonely2*5/8)))
            mask1(i,k)=((vel(i,k).EQ.nan).AND.(((vel(i -1,k).EQ.nan)
     $           +(vel(i+1,k).EQ.nan)+(vel(i+1,k-1).EQ.nan)
     $           +(vel(i,k-1).EQ.nan)+(vel(i-1,k-1).EQ.nan)).LT
     $           .(lonely2*5/8)))
         ENDDO

         DO i=1,mm
            DO j=1,nn
               IF (mask1(i,j).EQ.1) THEN
                  vel(i,j)=nan
               ENDIF
            ENDDO
         ENDDO

      ELSE                      !mm and/or nn too big

         PRINT*, "input matrix is too large. mm, nn="
         PRINT*, mm,nn
         PRINT*, "M, N="
         PRINT*, M,N
         DO i=1,mm
            DO j=1,nn
               vel(i,j)=v(i,j)
            ENDDO
         ENDDO
         
      ENDIF


      return
      end
      

